//
//  XSPEC12  November 2003
//
//
        
#include <XSUser/Global/Global.h>
#include <XSUser/Global/XSGlobal.h>
#include <XSUtil/Utils/XSutility.h>
#include <XSFit/Fit/Fit.h>
#include <XSFit/Fit/StatMethod.h>
#include <XSModel/GlobalContainer/DataContainer.h>
#include <XSModel/Data/SpectralData.h>
#include <XSModel/Data/BackCorr/Background.h>
#include <XSstreams.h>
#include <XSContainer.h>
#include <xsTypes.h>


int
XSGlobal::xsWeight(ClientData cdata,Tcl_Interp* tclInterp,int objc, Tcl_Obj* CONST  objv[])
{
        using namespace XSContainer;
        string cmd = string(static_cast<char*>(cdata));
        string arg;
        bool isChanged = false;
        if (objc == 1 || (arg = Tcl_GetString(objv[1])) == "?")
        {
            printDocs(cmd.c_str(),"?");
        } 
        else
        {
           // globalData's m_weight stores what the user has selected as
           // the weighting, either through their init file or this command.
           // DataUtility::statWeight holds the scheme that is actually applied 
           // to the variance.  These may differ if the chosen statistic
           // doesn't allow the selected weighting.
           
            Weight* tmpTest = StatMethod::weightScheme(arg);
            if (!tmpTest)
            {
               tcerr << "*** Unrecognized weighting scheme: " << arg << std::endl;
               printDocs(cmd.c_str(),"?");
               tcerr << "Current selection: " << globalData->weight() << std::endl;
               return globalData->autoSave(TCL_ERROR);   
            }
            globalData->weight(tmpTest->name());
            
            const string startingWeight = DataUtility::statWeight().name();            
            const string GEHRELS("gehrels");
            if (GEHRELS.find(XSutility::lowerCase(arg)) == 0)
            {
               tcout << "Warning: this weight scheme is only valid for Poisson data." 
                  << std::endl;
            }
            // The stat method doesn't necessarily set it to the arg setting,
            // ie. cstat and anything non-standard. 
            fit->statMethod()->setWeight(arg);
            if (startingWeight != DataUtility::statWeight().name())
               isChanged = true;  
        }
        
        if (isChanged)
        {
           size_t nSpec = datasets->numberOfSpectra();
           for (size_t i=0; i<nSpec; ++i)
           {
              SpectralData* sd = datasets->lookup(i+1);
              if (sd)
              {
                 RealArray norm = sd->areaScale()*sd->exposureTime();
                 if (Background* bkg = sd->getBackground())
                 {
                    RealArray bkgNorm = bkg->data()->areaScale()*bkg->data()->exposureTime();
                    bkgNorm *= bkg->data()->backgroundScale()/sd->backgroundScale();
                    DataUtility::statWeight()(*(bkg->data()), bkgNorm);
                 }
                 DataUtility::statWeight()(*sd, norm);
              }
           }
           fit->Update();
        }
        tcout << std::flush;
        return globalData->autoSave(TCL_OK);
}
